@@ -174,29 +174,20 @@ static void _gnutls_extension_list_add( gnutls_session session, uint16 type) {
 	}
 }
 
-int _gnutls_gen_extensions( gnutls_session session, opaque* data, size_t data_size) 
-{
+int _gnutls_gen_extensions( gnutls_session session, opaque** data) {
 int next, size;
 uint16 pos=0;
-opaque *sdata;
-int sdata_size;
+opaque sdata[1024];
+int sdata_size = sizeof(sdata);
 ext_send_func ext_send;
 
 
-	if (data_size < 2) {
-		gnutls_assert();
-		return GNUTLS_E_INTERNAL_ERROR;
-	}
-
-	/* allocate enough data for each extension.
-	 */
-	sdata_size = data_size;
-	sdata = gnutls_malloc( sdata_size);
-	if (sdata == NULL) {
+	(*data) = gnutls_malloc(2); /* allocate size for size */
+	if ((*data)==NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
-	
+
 	pos+=2;
 	next = MAX_EXT_TYPES; /* maximum supported extensions */
 	do {
@@ -205,21 +196,21 @@ ext_send_func ext_send;
 		if (ext_send == NULL) continue;
 		size = ext_send( session, sdata, sdata_size);
 		if (size > 0) {
-			if (data_size < pos+(size_t)size+4) {
+			(*data) = gnutls_realloc_fast( (*data), pos+size+4);
+			if ((*data)==NULL) {
 				gnutls_assert();
-				gnutls_free(sdata);
-				return GNUTLS_E_INTERNAL_ERROR;
+				return GNUTLS_E_MEMORY_ERROR;
 			}
 
 			/* write extension type */
-			_gnutls_write_uint16( next, &data[pos]);
+			_gnutls_write_uint16( next, &(*data)[pos]);
 			pos+=2;
 			
 			/* write size */
-			_gnutls_write_uint16( size, &data[pos]);
+			_gnutls_write_uint16( size, &(*data)[pos]);
 			pos+=2;
 			
-			memcpy( &data[pos], sdata, size);
+			memcpy( &(*data)[pos], sdata, size);
 			pos+=size;
 			
 			/* add this extension to the extension list
@@ -230,7 +221,7 @@ ext_send_func ext_send;
 				_gnutls_extension_get_name(next));
 		} else if (size < 0) {
 			gnutls_assert();
-			gnutls_free(sdata);
+			gnutls_free(*data); *data = NULL;
 			return size;
 		}
 		
@@ -239,13 +230,13 @@ ext_send_func ext_send;
 	size = pos;
 	pos-=2; /* remove the size of the size header! */
 
-	_gnutls_write_uint16( pos, data);
+	_gnutls_write_uint16( pos, (*data));
 
 	if (size==2) { /* empty */
 		size = 0;
+		gnutls_free(*data);
+		(*data) = NULL;
 	}
-	
-	gnutls_free( sdata);
 	return size;
 
 }
