@@ -181,26 +181,9 @@ const ASN1_ARRAY_TYPE pkix_asn1_tab[]={
   {0,4104,"1"},
   {"cRLIssuer",536895490,"GeneralNames"},
   {0,4104,"2"},
-  {"DistributionPointName",1610612754,0},
-  {"fullName",1610620930,"GeneralNames"},
+  {"DistributionPointName",1610612741,0},
+  {"fullName",536879106,"GeneralNames"},
   {0,4104,"0"},
-  {"nameRelativeToCRLIssuer",536879106,"RelativeDistinguishedName"},
-  {0,4104,"1"},
-  {"WritableCRLDistributionPoints",1612709899,0},
-  {"MAX",1074266122,"1"},
-  {0,2,"WritableDistributionPoint"},
-  {"WritableDistributionPoint",1610612741,0},
-  {"distributionPoint",1610637314,"WriteableDistributionPointName"},
-  {0,4104,"0"},
-  {"reasons",1610637314,"ReasonFlags"},
-  {0,4104,"1"},
-  {"cRLIssuer",536895490,"GeneralNames"},
-  {0,4104,"2"},
-  {"WriteableDistributionPointName",1610612754,0},
-  {"fullName",1610620941,0},
-  {0,4104,"0"},
-  {"nameRelativeToCRLIssuer",536879106,"RelativeDistinguishedName"},
-  {0,4104,"1"},
   {"ReasonFlags",1610874886,0},
   {"unused",1073741825,"0"},
   {"keyCompromise",1073741825,"1"},
