@@ -19,3 +19,7 @@ int gnutls_x509_crt_list_verify( gnutls_x509_crt* cert_list, int cert_list_lengt
 	gnutls_x509_crl* CRL_list, int CRL_list_length, 
 	unsigned int flags, unsigned int *verify);
 
+int _gnutls_x509_verify_signature( const gnutls_datum* tbs,
+	const gnutls_datum* signature, gnutls_x509_crt issuer);
+int _gnutls_x509_privkey_verify_signature( const gnutls_datum* tbs,
+	const gnutls_datum* signature, gnutls_x509_privkey issuer);
